## example of binary categorical data
## fictional AE data

modelName <- "binaryExample" # root names of model file

if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  Sys.setenv(HOME = file.path("C:/Users", Sys.info()["user"]))
}else{
  if(.Platform$OS.type == "windows")
    Sys.setenv(HOME = substr(R.home(), 1, 2))
}

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") ## change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  bugsDir <- file.path(Sys.getenv("HOME"), "WinBUGS14")
}else{
  if(.Platform$OS.type == "windows")
    bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
}
wineBin <- "/opt/local/bin" ## directory containing wine and winepath programs.
                            ## only relevant on unix or Mac OS X

setwd(exampleDir)

library(R2WinBUGS)
library(coda)
library(lattice)
library(locfit)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) ## not required but assures repeatable results

## get data files
xdata = read.csv("ae.data.csv",as.is=T)

################################################################################################
## exploratory plots

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"EdaPlots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

fit = locfit(ae~dose+as.factor(gender),xdata,link="logit")
x1 = scb(fit,ev=lfgrid(c(50,2)),link="logit",simul=F)
pred = rbind(data.frame(dose=x1$xev[,1],gender=x1$xev[,2],ae=logit.inv(x1$coef)),
	data.frame(dose=x1$xev[,1],gender=x1$xev[,2],ae=logit.inv(x1$lower)),
	data.frame(dose=x1$xev[,1],gender=x1$xev[,2],ae=logit.inv(x1$upper)))
pred$type = rep(c("mean","lower","upper"),ea=length(x1$coef))
x1 = aggregate(xdata["ae"],list(dose=xdata$dose,gender=xdata$gender),mean)
x1$dose = as.numeric(as.character(x1$dose))
x1$gender = as.numeric(as.character(x1$gender))
x1$type =rep("observed",nrow(x1))
pred=rbind(x1,pred)

print(xyplot(ae~dose|factor(gender,labels=c("male","female")),pred,groups=type,
	panel=panel.superpose,type=c("l","l","p","l"),lty=c(3,1,0,3),col=c("red","blue","black","red"),
	pch=c(NA,NA,19,NA),lwd=2,cex=0.8,scales=list(cex=0.8),
	xlab=list(label="dose",cex=1),ylab=list(label="fraction of patients with AE",
	cex=1),par.strip.text=list(cex=1),
	strip = function(...) strip.default(..., style = 1)),
	position=c(0,0.475,0.5,1),more=T)

fit = locfit(ae~age+as.factor(dose),xdata,link="logit")
x1 = scb(fit,ev=dat(),link="logit",simul=F)
pred = rbind(data.frame(age=x1$xev[,1],dose=x1$xev[,2],ae=logit.inv(x1$coef)),
	data.frame(age=x1$xev[,1],dose=x1$xev[,2],ae=logit.inv(x1$lower)),
	data.frame(age=x1$xev[,1],dose=x1$xev[,2],ae=logit.inv(x1$upper)))
pred$type = rep(c("mean","lower","upper"),ea=length(x1$coef))
x1 = xdata[c("age","dose","ae")]
x1$type =rep("observed",nrow(x1))
pred = rbind(x1,pred)
pred = pred[order(pred$dose,pred$age),]

print(xyplot(ae~age|as.factor(dose),pred,groups=type,
	panel=function(x,y,subscripts,groups,...){
		y[groups[subscripts]=="observed"] = jitter(y[groups[subscripts]=="observed"],amount=0.025)
		panel.superpose(x,y,subscripts,groups,...)
	},type=c("l","l","p","l"),lty=c(3,1,0,3),col=c("red","blue","black","red"),
	pch=c(NA,NA,19,NA),lwd=2,cex=0.5,scales=list(cex=0.8),
	xlab=list(label="age",cex=1),ylab=list(label="fraction of patients with AE",
	cex=1),par.strip.text=list(cex=1),
	strip = function(...) strip.default(..., style = 1)),
	position=c(0,0,0.5,0.525),more=T)

fit = locfit(ae~weight+as.factor(dose),xdata,link="logit")
x1 = scb(fit,ev=dat(),link="logit",simul=F)
pred = rbind(data.frame(weight=x1$xev[,1],dose=x1$xev[,2],ae=logit.inv(x1$coef)),
	data.frame(weight=x1$xev[,1],dose=x1$xev[,2],ae=logit.inv(x1$lower)),
	data.frame(weight=x1$xev[,1],dose=x1$xev[,2],ae=logit.inv(x1$upper)))
pred$type = rep(c("mean","lower","upper"),ea=length(x1$coef))
x1 = xdata[c("weight","dose","ae")]
x1$type =rep("observed",nrow(x1))
pred = rbind(x1,pred)
pred = pred[order(pred$dose,pred$weight),]

print(xyplot(ae~weight|as.factor(dose),pred,groups=type,
	panel=function(x,y,subscripts,groups,...){
		y[groups[subscripts]=="observed"] = jitter(y[groups[subscripts]=="observed"],amount=0.025)
		panel.superpose(x,y,subscripts,groups,...)
	},type=c("l","l","p","l"),lty=c(3,1,0,3),col=c("red","blue","black","red"),
	pch=c(NA,NA,19,NA),lwd=2,cex=0.5,scales=list(cex=0.8),
	xlab=list(label="weight",cex=1),ylab=list(label="fraction of patients with AE",
	cex=1),par.strip.text=list(cex=1),
	strip = function(...) strip.default(..., style = 1)),
	position=c(0.5,0,1,0.525),more=F)

dev.off()

################################################################################################

## create WinBUGS data set
bugsdata = list(
	nobs=nrow(xdata),
	dose=xdata$dose,
	age=xdata$age,
	weight=xdata$weight,
	gender=xdata$gender,  # male = 0; female = 1
	ae=xdata$ae
)

## create initial estimates
bugsinit = function() list(
	theta=rnorm(5,c(logit(0.05),0.1,0.05,-0.015,-0.7),4*abs(c(logit(0.05),0.1,0.05,-0.015,-0.7)))
)

## Specify the variables for which you want history and density plots
parametersToPlot <- c("theta")

## Additional variables to monitor
otherRVs <- c("p.ae","ae.pred")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)


################################################################################################
# run WinBUGS

n.chains = 3
n.iter = 10000
n.burnin = 4000
n.thin = 5
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = TRUE, debug = FALSE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory

save.model(bugs.fit, modelName)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- bugs.fit$sims.array
posterior <- array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
                   dimnames=list(NULL,dimnames(sims.array)[[3]]))

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
## posterior predictive distributions

## prediction of future observations in the same subjects, i.e., posterior predictions
## conditioned on observed data from the same subject

pred = posterior[,grep("ae.pred\\[",dimnames(posterior)[[2]])]

x1 = aggregate(xdata["ae"],list(dose=xdata$dose,gender=xdata$gender),mean)
x1$dose = as.numeric(as.character(x1$dose))
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$ae = as.vector(t(sapply(tapply(1:nrow(xdata),list(dose=xdata$dose,gender=xdata$gender),function(i,pred)
	apply(pred[,i],1,mean),pred=pred),quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(ae~dose,x1[x1$type=="observed",],groups=gender,panel=panel.superpose,
	type="b",col=c("blue","pink3"),pch=19,lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose",cex=1.2),ylab=list(label="fraction of patients with AE",
	cex=1.2),par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	key=list(x=0.4,y=0.8,text=list(labels=c("male","female"),cex=1.2),lines=list(type="b",
	col=c("blue","pink3"),pch=19,lwd=2,cex=1)))

xyplot(ae~dose|factor(gender,labels=c("male","female")),x1,groups=type,
	panel=panel.superpose,type=c("l","l","l","p"),lty=c(3,3,1,0),col=c(2,2,4,1),
	pch=c(NA,NA,NA,19),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose",cex=1.2),ylab=list(label="fraction of patients with AE",
	cex=1.2),par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

dev.off()
