# example of count data
# fictional seizure data

modelName <- "countExample" # root names of model file

if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  Sys.setenv(HOME = file.path("C:/Users", Sys.info()["user"]))
}else{
  if(.Platform$OS.type == "windows")
    Sys.setenv(HOME = substr(R.home(), 1, 2))
}

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") ## change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  bugsDir <- file.path(Sys.getenv("HOME"), "WinBUGS14")
}else{
  if(.Platform$OS.type == "windows")
    bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
}
wineBin <- "/opt/local/bin" ## directory containing wine and winepath programs.
                            ## only relevant on unix or Mac OS X

setwd(exampleDir)

library(R2WinBUGS)
library(coda)
library(lattice)
library(gdata)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) ## not required but assures repeatable results

## get data files
dataCount = read.csv("seizureCount.csv",as.is=T)

################################################################################################
## exploratory plots

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"EdaPlots%03d.pdf", sep = "")),
    width = 8, height = 6, onefile = F)

dataCount$seizureCount2 = ifelse(dataCount$seizureCount > 50, 50, dataCount$seizureCount)
countFreq = aggregate(1:nrow(dataCount), 
	list(dose=dataCount$dose, time=dataCount$time, seizureCount=dataCount$seizureCount), length)
countFreq = merge(countFreq, 
	expand.grid(dose=unique(dataCount$dose), time=unique(dataCount$time), 
	seizureCount=1:max(dataCount$seizureCount)), all=T)
countFreq = countFreq[order(countFreq$seizureCount, countFreq$dose, countFreq$time),]
countFreq$x = ifelse(is.na(countFreq$x), 0, countFreq$x)

countMax = 50 
barchart(x ~ seizureCount | as.factor(dose) * as.factor(time), countFreq, horizontal=F, origin=0,
	panel=function(x,y,...){
		panel.barchart(x,y,...)
		xm1 = as.numeric(as.character(x)) - 1
		gmean = sum(xm1*y)/sum(y)
		gvar = sum((xm1-gmean)^2*y)/(sum(y)-1)
		gmean = signif(gmean,3)
		gvar = signif(gvar,3)
		panel.text(20,11,paste("mean =",gmean,
			"\nvariance =",gvar), cex=0.7)
		panel.lines(1+(0:countMax), sum(y)*dpois(0:countMax,gmean),
			col="black", lwd=2)
		panel.text(25, 7, paste("poisson(", gmean, ")",sep=""), adj=c(0,0.5), cex=0.7)
		panel.arrows(25, 7, 21, sum(y)*dpois(20,gmean), lwd=2, length = 0.1)
	},
	xlim = c(0, countMax),
	cex=1,scales=list(cex=0.8, x=list(at=seq(1,countMax+10,by=10),
	labels=seq(0,countMax+1,by=10))),
	xlab=list(label="number of seizures per 4 weeks",cex=1.2),
	ylab=list(label="number of patients",cex=1.2),
	par.strip.text=list(cex=0.8),
	strip = function(...) strip.default(..., style = 1))
	
x1 = aggregate(dataCount["seizureCount"], list(dose=dataCount$dose, time=dataCount$time), mean)
x1$n = as.vector(tapply(dataCount$seizureCount,list(dose=dataCount$dose, time=dataCount$time),length))
x1$sd = as.vector(tapply(dataCount$seizureCount,list(dose=dataCount$dose, time=dataCount$time),sd))
x1$lci = x1$seizureCount - 2*x1$sd/sqrt(x1$n)
x1$uci = x1$seizureCount + 2*x1$sd/sqrt(x1$n)
xyplot(seizureCount~dose | as.factor(time),data=x1,
	ly=x1$lci,uy=x1$uci,
	prepanel=prepanel.ci,panel=panel.ci,
	type="b",
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="mean number of seizures per 4 weeks",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))
x1 = aggregate(dataCount["seizureCount"], list(dose=dataCount$dose, time=dataCount$time), median)
xyplot(seizureCount~dose | as.factor(time),data=x1,
	type="b",
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="dose",cex=1.2),
	ylab=list(label="median number of seizures per 4 weeks",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

dev.off()

################################################################################################

## create WinBUGS data set

bugsdata = list(
	nObs = nrow(dataCount),
	nPatients = length(unique(dataCount$patient)),
	patient = dataCount$patient,
	time = dataCount$time,
	dose = dataCount$dose,
	nSeizure = dataCount$seizureCount
)

## create initial estimates
bugsinit = function() list(
		logBase = rnorm(1, log(10), 0.5),
		logP1 = rnorm(1, log(1 - (-0.1)), 0.2),
		logP2 = rnorm(1, log(1 - (-0.1) - 0.3), 0.2),
		logED50 = rnorm(1, log(100), 0.5),
		omega = exp(rnorm(1, log(1), 0.5))
	)

## Specify the variables for which you want history and density plots
parametersToPlot <- c("base", "Eplacebo", "Emax", "ED50", "omega")

## Additional variables to monitor
otherRVs <- c("nSeizureCond", "nSeizurePred")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
## run WinBUGS

n.chains = 3
n.iter = 50000
n.burnin = 10000
n.thin = 25
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = TRUE, debug = FALSE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory

save.model(bugs.fit, modelName)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- bugs.fit$sims.array
posterior <- array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
                   dimnames=list(NULL,dimnames(sims.array)[[3]]))

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
## posterior predictive distributions

## prediction of future observations in the same subjects, i.e., posterior predictions of the
## observed" fraction of patients reporting each HRQOL score in the same group of patients

pred = posterior[,grep("nSeizureCond\\[",dimnames(posterior)[[2]])]

x1 = dataCount
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$seizureCount = as.vector(t(apply(pred,2,quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

doses = sort(unique(x1$dose))
for(dose in doses){
  print(xyplot(seizureCount~time|as.factor(patient),
               x1[x1$dose==dose,],groups=type,layout=c(5,5),
               panel=panel.superpose,type=c("l","l","l","p"),lty=c(3,3,1,0),col=c(2,2,4,1),
               pch=c(NA,NA,NA,19),lwd=2,cex=0.5,
               scales=list(cex=1, y=list(relation="free")),
               xlab=list(label="time (w)",cex=1.2),
               ylab=list(label="",cex=1.2),par.strip.text=list(cex=1),
               strip = function(...) strip.default(..., style = 1),
               main=list(label=paste(dose," mg/d",sep=""),cex=1.5),
               sub = "individual predictions"))
}

x1 = aggregate(dataCount["seizureCount"], 
	list(dose=dataCount$dose, time=dataCount$time), mean)
x1$dose = as.numeric(as.character(x1$dose))
x1$time = as.numeric(as.character(x1$time))
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$seizureCount = as.vector(t(sapply(tapply(1:nrow(dataCount),
	list(dose=dataCount$dose, time=dataCount$time),
	function(i,pred) apply(pred[,i],1,mean),pred=pred),quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(seizureCount~dose | as.factor(time),x1,groups=type,
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),
       col=c("blue","blue","red","black"),pch=c(rep(NA,3),19),
       lwd=2,cex=1,scales=list(cex=1),
       xlab=list(label="dose (mg/d)",cex=1.2),
       ylab=list(label="mean number of seizures",cex=1.2),
       main = "individual predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

xyplot(seizureCount~time | as.factor(dose),x1,groups=type,
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),
       col=c("blue","blue","red","black"),pch=c(rep(NA,3),19),
       lwd=2,cex=1,scales=list(cex=1),
       xlab=list(label="dose (mg/d)",cex=1.2),
       ylab=list(label="mean number of seizures",cex=1.2),
       main = "individual predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

x1 = aggregate(dataCount["seizureCount"], list(dose=dataCount$dose, time=dataCount$time), var)
x1$dose = as.numeric(as.character(x1$dose))
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$seizureCount = as.vector(t(sapply(tapply(1:nrow(dataCount),
	list(dose=dataCount$dose, time=dataCount$time),
	function(i,pred) apply(pred[,i],1,var),pred=pred),quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(seizureCount~dose | as.factor(time),x1,groups=type,
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),
       col=c("blue","blue","red","black"),pch=c(rep(NA,3),19),
       lwd=2,cex=1,scales=list(cex=1),
       xlab=list(label="dose (mg/d)",cex=1.2),
       ylab=list(label="variance of the number of seizures",cex=1.2),
       main = "individual predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

xyplot(seizureCount~time | as.factor(dose),x1,groups=type,
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),
       col=c("blue","blue","red","black"),pch=c(rep(NA,3),19),
       lwd=2,cex=1,scales=list(cex=1),
       xlab=list(label="dose (mg/d)",cex=1.2),
       ylab=list(label="variance of the number of seizures",cex=1.2),
       main = "individual predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

x1 = tapply(1:nrow(dataCount), list(dose=dataCount$dose, time=dataCount$time, 
	seizureCount=factor(dataCount$seizureCount, levels=0:max(dataCount$seizureCount))),
	length)
x1 =  ifelse(is.na(x1), 0, x1)
x1 = data.frame(dose = rep(dimnames(x1)[[1]],prod(dim(x1)[2:3])), 
	time = rep(rep(dimnames(x1)[[2]],ea=dim(x1)[1]),dim(x1)[3]),
	seizureCount = rep(dimnames(x1)[[3]],ea=prod(dim(x1)[1:2])),
	countFreq = as.vector(x1))
x1$type =rep("observed",nrow(x1))

x2 = rbind(x1,x1,x1)
x2$countFreq = as.vector(t(apply(apply(pred,1,function(pred,data){
		x1 = tapply(1:nrow(data), list(dose=data$dose, time=dataCount$time, 
			seizureCount=factor(pred, levels=0:max(data$seizureCount))),
			length)
		as.vector(ifelse(is.na(x1), 0, x1))
	},data=dataCount),1,quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

x1$seizureCount = as.numeric(as.character(x1$seizureCount))
x1$dose = as.numeric(as.character(x1$dose))
x1 = x1[order(x1$seizureCount, x1$dose),]

xyplot(countFreq~seizureCount|as.factor(dose)*reorder.factor(time),x1,groups=type,
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),
       col=c("blue","blue","red","black"),pch=c(rep(NA,3),20),
       lwd=2,cex=1,scales=list(cex=0.8),
       xlab=list(label="number of seizures",cex=1.2),
       ylab=list(label="number of patients",cex=1.2),
       main = "individual predictions",
       par.strip.text=list(cex=0.8),
       strip = function(...) strip.default(..., style = 1),
       xlim = c(0,50))

## prediction of future observations in new subjects

pred = posterior[,grep("nSeizurePred\\[",dimnames(posterior)[[2]])]

x1 = aggregate(dataCount["seizureCount"], 
	list(dose=dataCount$dose, time=dataCount$time), mean)
x1$dose = as.numeric(as.character(x1$dose))
x1$time = as.numeric(as.character(x1$time))
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$seizureCount = as.vector(t(sapply(tapply(1:nrow(dataCount),
	list(dose=dataCount$dose, time=dataCount$time),
	function(i,pred) apply(pred[,i],1,mean),pred=pred),quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(seizureCount~dose | as.factor(time),x1,groups=type,
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),
       col=c("blue","blue","red","black"),pch=c(rep(NA,3),19),
       lwd=2,cex=1,scales=list(cex=1),
       xlab=list(label="dose (mg/d)",cex=1.2),
       ylab=list(label="mean number of seizures",cex=1.2),
       main = "population predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

xyplot(seizureCount~time | as.factor(dose),x1,groups=type,
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),
       col=c("blue","blue","red","black"),pch=c(rep(NA,3),19),
       lwd=2,cex=1,scales=list(cex=1),
       xlab=list(label="time (w)",cex=1.2),
       ylab=list(label="mean number of seizures",cex=1.2),
       main = "population predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

x1 = aggregate(dataCount["seizureCount"], list(dose=dataCount$dose, time=dataCount$time), var)
x1$dose = as.numeric(as.character(x1$dose))
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$seizureCount = as.vector(t(sapply(tapply(1:nrow(dataCount),
	list(dose=dataCount$dose, time=dataCount$time),
	function(i,pred) apply(pred[,i],1,var),pred=pred),quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(seizureCount~dose | as.factor(time),x1,groups=type,
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),
       col=c("blue","blue","red","black"),pch=c(rep(NA,3),19),
       lwd=2,cex=1,scales=list(cex=1),
       xlab=list(label="dose (mg/d)",cex=1.2),
       ylab=list(label="variance of the number of seizures",cex=1.2),
       main = "population predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

xyplot(seizureCount~time | as.factor(dose),x1,groups=type,
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),
       col=c("blue","blue","red","black"),pch=c(rep(NA,3),19),
       lwd=2,cex=1,scales=list(cex=1),
       xlab=list(label="time(w)",cex=1.2),
       ylab=list(label="variance of the number of seizures",cex=1.2),
       main = "population predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

x1 = tapply(1:nrow(dataCount), list(dose=dataCount$dose, time=dataCount$time, 
	seizureCount=factor(dataCount$seizureCount, levels=0:max(dataCount$seizureCount))),
	length)
x1 =  ifelse(is.na(x1), 0, x1)
x1 = data.frame(dose = rep(dimnames(x1)[[1]],prod(dim(x1)[2:3])), 
	time = rep(rep(dimnames(x1)[[2]],ea=dim(x1)[1]),dim(x1)[3]),
	seizureCount = rep(dimnames(x1)[[3]],ea=prod(dim(x1)[1:2])),
	countFreq = as.vector(x1))
x1$type =rep("observed",nrow(x1))

x2 = rbind(x1,x1,x1)
x2$countFreq = as.vector(t(apply(apply(pred,1,function(pred,data){
		x1 = tapply(1:nrow(data), list(dose=data$dose, time=dataCount$time, 
			seizureCount=factor(pred, levels=0:max(data$seizureCount))),
			length)
		as.vector(ifelse(is.na(x1), 0, x1))
	},data=dataCount),1,quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

x1$seizureCount = as.numeric(as.character(x1$seizureCount))
x1$dose = as.numeric(as.character(x1$dose))
x1 = x1[order(x1$seizureCount, x1$dose),]

xyplot(countFreq~seizureCount|as.factor(dose)*reorder.factor(time),x1,groups=type,
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),
       col=c("blue","blue","red","black"),pch=c(rep(NA,3),20),
       lwd=2,cex=1,scales=list(cex=0.8),
       xlab=list(label="number of seizures",cex=1.2),
       ylab=list(label="number of patients",cex=1.2),
       main = "population predictions",
       par.strip.text=list(cex=0.8),
       strip = function(...) strip.default(..., style = 1),
       xlim = c(0,50))

dev.off()	
