# example of longitudinal binary categorical data
## fictional AE data (cough)

modelName <- "handsOn2" # root names of model file

if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  Sys.setenv(HOME = file.path("C:/Users", Sys.info()["user"]))
}else{
  if(.Platform$OS.type == "windows")
    Sys.setenv(HOME = substr(R.home(), 1, 2))
}

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") ## change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  bugsDir <- file.path(Sys.getenv("HOME"), "WinBUGS14")
}else{
  if(.Platform$OS.type == "windows")
    bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
}
wineBin <- "/opt/local/bin" ## directory containing wine and winepath programs.
                            ## only relevant on unix or Mac OS X

setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) ## not required but assures repeatable results

## get data files
dataProb2 = read.csv("ME2CoughData.csv",as.is=T)

################################################################################################
## exploratory plots

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"EdaPlots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

x1 = aggregate(dataProb2["cough"], list(dose = dataProb2$dose, time = dataProb2$time), sum)
x1$n = as.vector(tapply(dataProb2$cough,list(dose = dataProb2$dose, time = dataProb2$time),length))
x1$mean = x1$cough/x1$n
x2 = sapply(1:nrow(x1),function(i,y) binom.test(y$cough[i],y$n[i])$conf.int,y=x1)
x1$lci = x2[1,]
x1$uci = x2[2,]
xyplot(mean~dose,data=x1,groups=time,
	ly=x1$lci,uy=x1$uci,
	prepanel=prepanel.ci,panel=panel.superpose,panel.groups=panel.ci,
	type="b",col=c("black","blue","red","green","cyan","magenta"),pch="circle",
	key=list(text=list(label=paste(sort(unique(x1$time)),"weeks"),cex=1.2),columns=2,
	points=list(col=c("black","blue","red","green","cyan","magenta"),pch="circle"),
	lines=list(col=c("black","blue","red","green","cyan","magenta"))),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="dose (mg/d)",cex=1.2),
	ylab=list(label="fraction of patients reporting cough",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

xyplot(mean~time,data=x1,groups=dose,
	ly=x1$lci,uy=x1$uci,
	prepanel=prepanel.ci,panel=panel.superpose,panel.groups=panel.ci,
	type="b",col=c("black","blue","red","green","cyan","magenta"),pch="circle",
	key=list(text=list(label=paste(sort(unique(x1$dose)),"mg/d"),cex=1.2),columns=2,
	points=list(col=c("black","blue","red","green"),pch="circle"),
	lines=list(col=c("black","blue","red","green"))),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="time (weeks)",cex=1.2),
	ylab=list(label="fraction of patients reporting cough",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

xyplot(logit(mean)~dose,data=x1,groups=time,
	ly=logit(x1$lci),uy=logit(x1$uci),
	prepanel=prepanel.ci,panel=panel.superpose,panel.groups=panel.ci,
	type="b",col=c("black","blue","red","green","cyan","magenta"),pch="circle",
	key=list(text=list(label=paste(sort(unique(x1$time)),"weeks"),cex=1.2),columns=2,
	points=list(col=c("black","blue","red","green","cyan","magenta"),pch="circle"),
	lines=list(col=c("black","blue","red","green","cyan","magenta"))),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="dose (mg/d)",cex=1.2),
	ylab=list(label="logit(fraction of patients reporting cough)",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

xyplot(logit(mean)~time,data=x1,groups=dose,
	ly=logit(x1$lci),uy=logit(x1$uci),
	prepanel=prepanel.ci,panel=panel.superpose,panel.groups=panel.ci,
	type="b",col=c("black","blue","red","green","cyan","magenta"),pch="circle",
	key=list(text=list(label=paste(sort(unique(x1$dose)),"mg/d"),cex=1.2),columns=2,
	points=list(col=c("black","blue","red","green"),pch="circle"),
	lines=list(col=c("black","blue","red","green"))),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="time (weeks)",cex=1.2),
	ylab=list(label="logit(fraction of patients reporting cough)",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

dev.off()

################################################################################################

## create WinBUGS data set

## trick to create consecutive ID numbers
dataProb2$patID = as.numeric(as.factor(dataProb2$patient))
## calculate midpoint time of each interval. assumes data are time-ordered
dataProb2$tMid = as.vector(sapply(unique(dataProb2$patient),
	function(patient,data){
		x1 = data[data$patient==patient,]
		(x1$time + c(0,x1$time[-length(x1$time)])) / 2
	},data=dataProb2))
bugsdata = list(
	nobs = nrow(dataProb2),
	nPatients = length(unique(dataProb2$patient)),
	patient = dataProb2$patID,
	time = dataProb2$tMid,
	dose = dataProb2$dose,
	cough = dataProb2$cough
)

## create initial estimates

bugsinit = function() list(
	E0Mean = rnorm(1,mean(logit(x1$mean[x1$dose==0])),2*sd(logit(x1$mean[x1$dose==0]))),
	alpha = exp(rnorm(1,
		log((x1$mean[x1$dose==20&x1$time==4]-x1$mean[x1$dose==0&x1$time==4])/20),
		0.4)),
	logxTol50 = rnorm(1,log(20),0.4),
	logkTol = rnorm(1,log(log(2)/20),0.4),
	sigma = exp(rnorm(1,1,0.5))
)


## Specify the variables for which you want history and density plots
parametersToPlot <- c("E0Mean", "alpha", "xTol50", "kTol", "sigma")

## Additional variables to monitor
otherRVs <- c("E0", "coughCond", "coughPred")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
## run WinBUGS

n.chains = 3
n.iter = 10000
n.burnin = 4000
n.thin = 10
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = TRUE, debug = FALSE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory

save.model(bugs.fit, modelName)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- bugs.fit$sims.array
posterior <- array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
                   dimnames=list(NULL,dimnames(sims.array)[[3]]))

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
## posterior predictive distributions

## prediction of future observations in the same subjects, i.e., posterior predictions of the
## observed" fraction of patients with >=1 pulmonary exacerbation in the same group of patients

pred = posterior[,grep("coughCond\\[",dimnames(posterior)[[2]])]

x1 = aggregate(dataProb2["cough"],list(dose=dataProb2$dose,time=dataProb2$time),mean)
x1$dose = as.numeric(as.character(x1$dose))
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$cough = as.vector(t(sapply(tapply(1:nrow(dataProb2),list(dose=dataProb2$dose,time=dataProb2$time),
	function(i,pred) apply(pred[,i],1,mean),pred=pred),quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(cough~dose|as.factor(time),x1,groups=type,
	panel=panel.superpose,type=c(rep("l",3),"p"),lty=c(2,2,1,0),col=c("red","red","blue","black"),
	pch=c(rep(NA,3),19),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose (mg/d)",cex=1.2),
	ylab=list(label="fraction of patients reporting cough",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	ylim=c(0,1.1*max(x1$cough)))

xyplot(cough~time|as.factor(dose),x1,groups=type,
	panel=panel.superpose,type=c(rep("l",3),"p"),lty=c(2,2,1,0),col=c("red","red","blue","black"),
	pch=c(rep(NA,3),19),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="time (weeks)",cex=1.2),
	ylab=list(label="fraction of patients reporting cough",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	ylim=c(0,1.1*max(x1$cough)))

## prediction of future observations in new subjects, i.e., posterior predictions of the
## observed" fraction of patients with >=1 pulmonary exacerbation assuming the same number of patients
## with the same covariates

pred = posterior[,grep("coughPred\\[",dimnames(posterior)[[2]])]

x1 = aggregate(dataProb2["cough"],list(dose=dataProb2$dose,time=dataProb2$time),mean)
x1$dose = as.numeric(as.character(x1$dose))
x1$type =rep("observed",nrow(x1))
x2 = rbind(x1,x1,x1)
x2$cough = as.vector(t(sapply(tapply(1:nrow(dataProb2),list(dose=dataProb2$dose,time=dataProb2$time),
	function(i,pred) apply(pred[,i],1,mean),pred=pred),quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(cough~dose|as.factor(time),x1,groups=type,
	panel=panel.superpose,type=c(rep("l",3),"p"),lty=c(2,2,1,0),col=c("red","red","blue","black"),
	pch=c(rep(NA,3),19),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="dose (mg/d)",cex=1.2),
	ylab=list(label="fraction of patients reporting cough",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	ylim=c(0,1.1*max(x1$cough)))

xyplot(cough~time|as.factor(dose),x1,groups=type,
	panel=panel.superpose,type=c(rep("l",3),"p"),lty=c(2,2,1,0),col=c("red","red","blue","black"),
	pch=c(rep(NA,3),19),lwd=2,cex=1,scales=list(cex=1),
	xlab=list(label="time (weeks)",cex=1.2),
	ylab=list(label="fraction of patients reporting cough",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1),
	ylim=c(0,1.1*max(x1$cough)))

################################################################################################
## IIV random effect plots

x1 = posterior[,grep("E0\\[",dimnames(posterior)[[2]])]

x2 = apply(x1,2,median)

plots = qqhist(x2,label="median E0")
print(plots[[1]],position=c(0,0,0.525,1),more=T)
print(plots[[2]],position=c(0.475,0,1,1),more=F)

dev.off()	
