$PROB PD Model for Quality of Life Ordinal Scale

$INPUT  ID DOSE TIME DV
; DV = QOL  1=WORSE  2=UNCHANGED 3=BETTER

$DATA ME2QOLData.CSV IGNORE=@

$PRED
; ORDINAL CATEGORICAL RESPONSE DATA MODELED WITH CUMULATIVE LOGIT MODEL 
; (ALSO SEE AGRESTI, A. CATEGORICAL DATA ANALYSIS. 1990 WILEY. PP 322-331.)
; SCALE = 1, 2, 3 (WORSE, UNCHANGED, BETTER)

; cutpoint values (alpha[j] in Agresti notation)
 AL1 = THETA(1)    ;SCORE <=1
 AL2 = THETA(2)    ; AL2 + AL1 is cutpoint for SCORE <=2
 
;PLACEBO EFFECT MODEL
 BETA = THETA(3)
 EPBO = BETA*TIME

;DRUG EFFCT MODEL
 A = THETA(4)
 B = THETA(5)
 LOGK = THETA(6)
 K = EXP(LOGK)
 EDRG = DOSE*(A + B * (1 - EXP(-K*TIME)))

; logits for cummulative probabilities 
 LGT1 = AL1 - (EPBO + EDRG + ETA(1))
 LGT2 = LGT1 + AL2

; CUMMULATIVE PROBABILITIES
 CP1 = EXP(LGT1)/(1+EXP(LGT1)) ;P(Y<=1|X)
 CP2 = EXP(LGT2)/(1+EXP(LGT2)) ;P(Y<=2|X)
 CP3 = 1

;PROB FOR EACH CATEGORY
 P1 = CP1
 P2 = CP2 - CP1
 P3 = 1 - CP2

; PRED EXIT CODES TO AVOID NEGLIGIBLE PROBABILITY DURING PARAM SEARCH
IF(P1.LE.0.0001) EXIT 1 2
IF(P2.LE.0.0001) EXIT 1 3
IF(P3.LE.0.0001) EXIT 1 4

; FOR SIMULATION OF ORDINAL DATA
; IF(ICALL.EQ.4) THEN
;   CALL RANDOM (2,R)
;   IF(R.LE.CP1) DV=1
;   IF(R.GT.CP1.AND.R.LE.CP2) DV=2
;   IF(R.GT.CP2) DV=3
;   X2=R
; ENDIF

 IND1=0
 IND2=0
 IND3=0 
 IF (DV.EQ.1) IND1=1
 IF (DV.EQ.2) IND2=1
 IF (DV.EQ.3) IND3=1
 
Y= P1*IND1 + P2*IND2 + P3*IND3

; INITIAL ESTIMATES FROM bugsinit() in R
$THETA
 -1.32  ; AL1
 2.18   ; AL2
 -0.06 ; BETA
 -1.26 ; A
 1.47 ; B
 -3.36 ; LOG(K)

$OMEGA  
 4.88

$ESTIMATION MAX=9999 PRINT=1 METHOD=COND LAPLACE LIKE NOABORT MSFO=4.MSF
;$SIM (95465) (546 UNIFORM) ONLYSIM NOPRED SUB=100
$TABLE NOPRINT NOHEADER FILE=4.TAB
 ID ETA1 P1 P2 P3 CP1 CP2 DOSE TIME
