## example of longitudinal ordinal data
## fictional health-related quality of lofe (HRQOL) data

modelName <- "handsOn3Repar1" # root names of model file
exampleDir <- "handsOn3"

if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  Sys.setenv(HOME = file.path("C:/Users", Sys.info()["user"]))
}else{
  if(.Platform$OS.type == "windows")
    Sys.setenv(HOME = substr(R.home(), 1, 2))
}

courseDir <- file.path(Sys.getenv("HOME"), "MI255")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI255/MI255")
exampleDir <- file.path(courseDir, exampleDir)
toolsDir <- file.path(courseDir, "bugsTools") # change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(Sys.info()["nodename"] == "IP-0AC33FDF"){
  bugsDir <- file.path(Sys.getenv("HOME"), "WinBUGS14")
}else{
  if(.Platform$OS.type == "windows")
    bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
}
wineBin <- "/opt/local/bin" ## directory containing wine and winepath programs.
                            ## only relevant on unix or Mac OS X

setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
library(locfit)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) ## not required but assures repeatable results

## get data files
dataProb3 = read.csv("ME2QOLData.csv",as.is=T)

################################################################################################
## exploratory plots

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"EdaPlots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

x1 = rbind(
	aggregate(dataProb3["QOL"], list(dose = dataProb3$dose, time = dataProb3$time),
		function(x) frac(x==1)),
	aggregate(dataProb3["QOL"], list(dose = dataProb3$dose, time = dataProb3$time),
		function(x) frac(x==2)),
	aggregate(dataProb3["QOL"], list(dose = dataProb3$dose, time = dataProb3$time),
		function(x) frac(x==3))
	)
x1$stat = rep(c("F1","F2","F3"),ea=nrow(x1)/3)
x1$n = rep(as.vector(tapply(dataProb3$QOL,list(dose = dataProb3$dose, time = dataProb3$time),length)), 3)
x2 = sapply(1:nrow(x1),function(i,y) binom.test(y$QOL[i]*y$n[i],y$n[i])$conf.int,y=x1)
x1$lci = x2[1,]
x1$uci = x2[2,]

xyplot(QOL~time|as.factor(dose),data=x1,groups=stat,
	ly=x1$lci,uy=x1$uci,
	prepanel=prepanel.ci,panel=panel.superpose,panel.groups=panel.ci,
	type="b",col=c("red","black","green"),pch="circle",
	key=list(text=list(label=c("worse","unchanged","better"),cex=1.2),
	points=list(col=c("red","black","green"),pch="circle"),
	lines=list(col=c("red","black","green"))),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="time (weeks)",cex=1.2),
	ylab=list(label="fraction of patients",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

x1 = rbind(
	aggregate(dataProb3["QOL"], list(dose = dataProb3$dose, time = dataProb3$time),
		function(x) frac(x<=1)),
	aggregate(dataProb3["QOL"], list(dose = dataProb3$dose, time = dataProb3$time),
		function(x) frac(x<=2))
	)
x1$stat = rep(c("Flt1","Flt2"),ea=nrow(x1)/2)
x1$n = rep(as.vector(tapply(dataProb3$QOL,list(dose = dataProb3$dose, time = dataProb3$time),length)), 2)
x2 = sapply(1:nrow(x1),function(i,y) binom.test(y$QOL[i]*y$n[i],y$n[i])$conf.int,y=x1)
x1$lci = x2[1,]
x1$uci = x2[2,]

xyplot(QOL~time|as.factor(dose),data=x1,groups=stat,
	ly=x1$lci,uy=x1$uci,
	prepanel=prepanel.ci,panel=panel.superpose,panel.groups=panel.ci,
	type="b",col=c("red","blue"),pch="circle",
	key=list(text=list(label=c("worse (<= 1)","worse or unchanged (<= 2)"),cex=1.2),
	points=list(col=c("red","blue"),pch="circle"),
	lines=list(col=c("red","blue"))),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="time (weeks)",cex=1.2),
	ylab=list(label="fraction of patients",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

xyplot(logit(QOL)~time|as.factor(dose),data=x1,groups=stat,
	ly=logit(x1$lci),uy=logit(x1$uci),
	prepanel=prepanel.ci,panel=panel.superpose,panel.groups=panel.ci,
	type="b",col=c("red","blue"),pch="circle",
	key=list(text=list(label=c("worse (<= 1)","worse or unchanged (<= 2)"),cex=1.2),
	points=list(col=c("red","blue"),pch="circle"),
	lines=list(col=c("red","blue"))),
	cex=1.2,scales=list(cex=1.2),
	xlab=list(label="time (weeks)",cex=1.2),
	ylab=list(label="logit(fraction of patients)",cex=1.2),
	par.strip.text=list(cex=1.2),
	strip = function(...) strip.default(..., style = 1))

## Plot odds for QOL <= 1 and <=2 to assess compatibility with a
## rpoportional odds assumption

x1$odds <- x1$QOL / (1 - x1$QOL)

xyplot(log(odds)~time|as.factor(dose),data=x1,groups=stat,
       panel=panel.superpose,
       type="b",col=c("red","blue"),pch="circle",
       key=list(text=list(label=c("worse (<= 1)","worse or unchanged (<= 2)"),cex=1.2),
         points=list(col=c("red","blue"),pch="circle"),
         lines=list(col=c("red","blue"))),
       cex=1.2,scales=list(cex=1.2),
       xlab=list(label="time (weeks)",cex=1.2),
       ylab=list(label="log(odds)",cex=1.2),
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

xyplot(log(odds)~dose|as.factor(time),data=x1,groups=stat,
       panel=panel.superpose,
       type="b",col=c("red","blue"),pch="circle",
       key=list(text=list(label=c("worse (<= 1)","worse or unchanged (<= 2)"),cex=1.2),
         points=list(col=c("red","blue"),pch="circle"),
         lines=list(col=c("red","blue"))),
       cex=1.2,scales=list(cex=1.2),
       xlab=list(label="dose (mg/d)",cex=1.2),
       ylab=list(label="log(odds)",cex=1.2),
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

xyplot(log(odds)~as.numeric(as.factor(stat))|as.factor(time),data=x1,groups=dose,
       panel=panel.superpose,
       type="b",col=c("red","blue"),pch="circle",
       key=list(text=list(label=c("dose = 0","dose = 40"),cex=1.2),
         points=list(col=c("red","blue"),pch="circle"),
         lines=list(col=c("red","blue"))),
       cex=1.2,
       scales=list(cex=1.2,
         x = list(at = c(1, 2), labels = c("QOL <= 1", "QOL <= 2"), rot = 45)),
       xlab=NULL,
       ylab=list(label="log(odds)",cex=1.2),
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

xyplot(log(odds)~as.numeric(as.factor(stat))|as.factor(dose),data=x1,groups=time,
       panel=panel.superpose,
       type="b",col=1:6,pch="circle",
       key=list(text=list(label=as.character(sort(unique(x1$time))),cex=1.2),
         points=list(col=1:6,pch="circle"),
         lines=list(col=1:6), columns = 2),
       cex=1.2,
       scales=list(cex=1.2,
         x = list(at = c(1, 2), labels = c("QOL <= 1", "QOL <= 2"), rot = 45)),
       xlab=NULL,
       ylab=list(label="log(odds)",cex=1.2),
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

dev.off()

################################################################################################

## create WinBUGS data set

## trick to create consecutive ID numbers
dataProb3$patID = as.numeric(as.factor(dataProb3$patient))

bugsdata = list(
	nobs = nrow(dataProb3),
	nPatients = length(unique(dataProb3$patient)),
	patient = dataProb3$patID,
	time = dataProb3$time,
	Idrug = as.numeric(dataProb3$dose > 0),
	qol = dataProb3$QOL
)

## create initial estimates
bugsinit = function() list(
	alpha = rnorm(2, c(-1.3, 2), 1),
	beta = rnorm(1, 0, 1/24),
	a = rnorm(1, -1.6, 1),
	bStar = rnorm(1, 2.5, 1),
	logk = rnorm(1, log(log(2)/8), 0.5),
	sigma = exp(rnorm(1, 1, 0.5))
)

## Specify the variables for which you want history and density plots
parametersToPlot <- c("alpha", "beta", "a", "b", "bStar", "k", "sigma")

## Additional variables to monitor
otherRVs <- c("eta", "qolCond", "qolPred")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
## run WinBUGS

n.chains = 3
n.iter = 10000
n.burnin = 4000
n.thin = 10
##n.iter = 50000
##n.burnin = 10000
##n.thin = 25
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = TRUE, debug = FALSE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory

save.model(bugs.fit, modelName)
## load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots

sims.array <- bugs.fit$sims.array
posterior <- array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
                   dimnames=list(NULL,dimnames(sims.array)[[3]]))

################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
    width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 <- sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
                                   paste("^",parametersToPlot,"\\[",sep="")),
                                 grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable <- parameter.plot.table(x1)
write.csv(signif(ptable,3),paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
## posterior predictive distributions

## prediction of future observations in the same subjects, i.e., posterior predictions of the
## observed" fraction of patients reporting each HRQOL score in the same group of patients

pred = posterior[,grep("qolCond\\[",dimnames(posterior)[[2]])]

x1 = rbind(
  aggregate(dataProb3["QOL"], list(dose = dataProb3$dose, time = dataProb3$time),
            function(x) frac(x==1)),
  aggregate(dataProb3["QOL"], list(dose = dataProb3$dose, time = dataProb3$time),
            function(x) frac(x==2)),
  aggregate(dataProb3["QOL"], list(dose = dataProb3$dose, time = dataProb3$time),
            function(x) frac(x==3))
  )
x1$stat = rep(c("F1","F2","F3"),ea=nrow(x1)/3)
x1$type = rep("observed", nrow(x1))

x2 = rbind(x1,x1,x1)
x2$QOL = as.vector(t(apply(
  apply(pred, 1, function(x,data) 
        c(
          as.vector(tapply(x, list(dose = data$dose, time = data$time),
                           function(x) frac(x==1))),
          as.vector(tapply(x, list(dose = data$dose, time = data$time),
                           function(x) frac(x==2))),
          as.vector(tapply(x, list(dose = data$dose, time = data$time),
                           function(x) frac(x==3)))
          ),
        data = dataProb3),
  1, quantile, probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(QOL~time|as.factor(dose),data=x1,groups=paste(stat,type),
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),col=rep(c("red","black","green"),ea=4),
       pch=c(rep(NA,3),19),
       key=list(text=list(label=c("worse","unchanged","better"),cex=1.2),
         points=list(col=c("red","black","green"),pch=19,cex=1.2),
         lines=list(col=c("red","black","green"))),
       cex=1.2,scales=list(cex=1.2),
       xlab=list(label="time (weeks)",cex=1.2),
       ylab=list(label="fraction of patients",cex=1.2),
       main = "individual predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

xyplot(logit(QOL)~time|as.factor(dose),data=x1,groups=paste(stat,type),
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),col=rep(c("red","black","green"),ea=4),
       pch=c(rep(NA,3),19),
       key=list(text=list(label=c("worse","unchanged","better"),cex=1.2),
         points=list(col=c("red","black","green"),pch=19,cex=1.2),
         lines=list(col=c("red","black","green"))),
       cex=1.2,scales=list(cex=1.2),
       xlab=list(label="time (weeks)",cex=1.2),
       ylab=list(label="logit(fraction of patients)",cex=1.2),
       main = "individual predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

## prediction of future observations in new subjects, i.e., posterior predictions of the
## observed" fraction of patients reporting each HRQOL score assuming the same number of patients
## with the same covariates

pred = posterior[,grep("qolPred\\[",dimnames(posterior)[[2]])]

x1 = rbind(
  aggregate(dataProb3["QOL"], list(dose = dataProb3$dose, time = dataProb3$time),
            function(x) frac(x==1)),
  aggregate(dataProb3["QOL"], list(dose = dataProb3$dose, time = dataProb3$time),
            function(x) frac(x==2)),
  aggregate(dataProb3["QOL"], list(dose = dataProb3$dose, time = dataProb3$time),
            function(x) frac(x==3))
  )
x1$stat = rep(c("F1","F2","F3"),ea=nrow(x1)/3)
x1$type = rep("observed", nrow(x1))

x2 = rbind(x1,x1,x1)
x2$QOL = as.vector(t(apply(
  apply(pred, 1, function(x,data) 
        c(
          as.vector(tapply(x, list(dose = data$dose, time = data$time),
                           function(x) frac(x==1))),
          as.vector(tapply(x, list(dose = data$dose, time = data$time),
                           function(x) frac(x==2))),
          as.vector(tapply(x, list(dose = data$dose, time = data$time),
                           function(x) frac(x==3)))
          ),
        data = dataProb3),
  1, quantile, probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1 = rbind(x1,x2)

xyplot(QOL~time|as.factor(dose),data=x1,groups=paste(stat,type),
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),col=rep(c("red","black","green"),ea=4),
       pch=c(rep(NA,3),19),
       key=list(text=list(label=c("worse","unchanged","better"),cex=1.2),
         points=list(col=c("red","black","green"),pch=19,cex=1.2),
         lines=list(col=c("red","black","green"))),
       cex=1.2,scales=list(cex=1.2),
       xlab=list(label="time (weeks)",cex=1.2),
       ylab=list(label="fraction of patients",cex=1.2),
       main = "population predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

xyplot(logit(QOL)~time|as.factor(dose),data=x1,groups=paste(stat,type),
       type=c(rep("l",3),"p"),lty=c(2,2,1,0),col=rep(c("red","black","green"),ea=4),
       pch=c(rep(NA,3),19),
       key=list(text=list(label=c("worse","unchanged","better"),cex=1.2),
         points=list(col=c("red","black","green"),pch=19,cex=1.2),
         lines=list(col=c("red","black","green"))),
       cex=1.2,scales=list(cex=1.2),
       xlab=list(label="time (weeks)",cex=1.2),
       ylab=list(label="logit(fraction of patients)",cex=1.2),
       main = "population predictions",
       par.strip.text=list(cex=1.2),
       strip = function(...) strip.default(..., style = 1))

################################################################################################
## IIV random effect plots

x1 = posterior[,grep("eta\\[",dimnames(posterior)[[2]])]

x2 = apply(x1,2,median)

plots = qqhist(x2,label="median eta")
print(plots[[1]],position=c(0,0,0.525,1),more=T)
print(plots[[2]],position=c(0.475,0,1,1),more=F)

dev.off()	
