$PROB PD Model for Coughing Count Data as f(Dose): PoissonGamma
   ; also know as Negative Binomial PDF

$INPUT ID DOSE DV TIME
    ; all time intervals are equal

$DATA ME2CoughCountData.csv  IGNORE=@

$PRED
; Negative Binomial PDF
   ; Requires expression for Gamma function
   ; Gergo Nemes closed form approximation of the Gamma function
   ; Stan's Library, Volume II, 31 Dec 2007
   ; www.ebyte.it/library/docs/math07/GammaApproximation.html
   ; E =  2.718281
   ; PI = 3.141592
   ; Gamma(Z) = (Z/E)**Z * (2*PI/Z)**0.5 * (1 + 1/15*Z**2)**(5/4*Z)
   
; Define Lambda (mean counts) as in Poisson
   LOGA = THETA(1)
   A = EXP(LOGA)
   LOGB = THETA(2)
   B = EXP(LOGB)
   LM = A + B * DOSE + ETA(1) ; ETA=0  Mean Counts

; Define Negative Binomial PDF
   E =  2.718281   ;constant
   PI = 3.141592   ;constant
   ODP=THETA(3)   ;Degree of over-dispersion (ODP=0 is basic Poisson Dist.)
   
; Nemes approximation to Gamma function 
   W = 1/ODP
   Z = DV + W
   GMZ = (Z/E)**Z * (2*PI/Z)**0.5 * (1 + 1/15*Z**2)**(5/4*Z)   ;Gamma function of Z
   GMW = (W/E)**W * (2*PI/W)**0.5 * (1 + 1/15*W**2)**(5/4*W)  ;Gamma function of W

; Negative Binomial PDF
   NGBN = GMZ/GMW * (1/(1+ODP*LM))**(W) * (LM/(W+LM))**DV

; Mean Counts is LM
; Variance of Counts
   VARC = LM + ODP*(LM**2)
   
   Y=NGBN

; initial estimates from bugsinit()
$THETA  
  -2.78 ; LOGA
  -6.8  ; LOGB
  (0, 0.001) ; ODP : Initial estimate close to zero (e.g. close to Poisson)

$OMEGA 0 FIX 

$EST PRINT=2 METH=1 LAPLACE LIKELIHOOD

$TABLE ONEHEADER NOPRINT FILE=NGBN.tab 
 ID A B LM VARC DV DOSE TIME